-- Author: Jarcast
-- DateCreated: 4/21/2023 6:24:02 PM
--==========================================================================================================================
-- INCLUDES
--==========================================================================================================================
include("FLuaVector.lua")
include("IconSupport.lua")
include("InstanceManager.lua")
--==========================================================================================================================
-- GLOBALS
--==========================================================================================================================
----------------------------------------------------------------------------------------------------------------------------
local Teams 					= Teams
local activePlayerID			= Game.GetActivePlayer()
local activePlayer				= Players[activePlayerID]
local activeTeamID				= activePlayer:GetTeam()
local activeTeam				= Teams[activeTeamID]
--==========================================================================================================================
-- UTILITIES
--==========================================================================================================================
-- UTILITIES
----------------------------------------------------------------------------------------------------------------------------
--Game_IsUsingMod
local CBPmodID = "d1b6328c-ff44-4b0d-aad7-c657f83610cd"
local CBOmodID = "8411a7a8-dad3-4622-a18e-fcc18324c799"
local CSDmodID = "eead0050-1e3f-4178-a91f-26cf1881ac39"
local EEraModID = "ce8aa614-7ef7-4a45-a179-5329869e8d6d"
local EECBPmodID  = "87b2deae-83cb-9b88-9017-79e741d3739c"

function Game_IsUsingMod(modID)
	for _, mod in pairs(Modding.GetActivatedMods()) do
		if mod.ID == modID then
			return true
		end
	end
	return false
end

local isUsingEEra = Game_IsUsingMod(EEraModID) or Game_IsUsingMod(EECBPmodID)
local isUsingCBP  = Game_IsUsingMod(CBPmodID)
local isUsingCBO  = Game_IsUsingMod(CBOmodID)
----------------------------------------------------------------------------------------------------------------------------
--HasTrait
function HasTrait(player, traitID)
	if Game_IsUsingMod(CBPmodID) then 
		return player:HasTrait(traitID)
	else
		local leaderType = GameInfo.Leaders[player:GetLeaderType()].Type
		local traitType  = GameInfo.Traits[traitID].Type
		for row in GameInfo.Leader_Traits("LeaderType = '" .. leaderType .. "' AND TraitType = '" .. traitType .. "'") do
			return true
		end
	end
	return false
end
-------------------------------------------------------------------------------------------------------------------------
-- GetRandom
function GetRandom(lower, upper)
    return Game.Rand((upper + 1) - lower, "") + lower
end
-------------------------------------------------------------------------------------------------------------------------
--IsTraitActive
local slotComputerID = SlotStatus["SS_COMPUTER"]
local slotTakenID = SlotStatus["SS_TAKEN"]
function IsTraitActive(traitID)
	for slotID = 0, GameDefines.MAX_MAJOR_CIVS-1, 1 do
		local slotPlayer = Players[slotID]
		local slotStatus = PreGame.GetSlotStatus(slotID)
		if (slotStatus == slotTakenID or slotStatus == slotComputerID) then
			if HasTrait(slotPlayer, traitID) then
				return true
			end
		end
	end
	return false
end
-------------------------------------------------------------------------------------------------------------------------
if Game then
	--Game.GetUserSetting
	function Game.GetUserSetting(type)
		for row in GameInfo.JFD_GlobalUserSettings("Type = '" .. type .. "'") do
			return row.Value
		end
	end
end
-------------------------------------------------------------------------------------------------------------------------
function IsInteger(n)
  return n==math.floor(n)
end
-------------------------------------------------------------------------------------------------------------------------
-- base converter
local floor,insert = math.floor, table.insert
function basen(n,b)
    n = floor(n)
    if not b or b == 10 then return tostring(n) end
    local digits = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    local t = {}
    local sign = ""
    if n < 0 then
        sign = "-"
    n = -n
    end
    repeat
        local d = (n % b) + 1
        n = floor(n / b)
        insert(t, 1, digits:sub(d,d))
    until n == 0
    return sign .. table.concat(t,"")
end
-------------------------------------------------------------------------------------------------------------------------
function Neirai_GetNearestCity(pPlayer, pPlot)
    local distance = 9999
    local cNearestCity = nil
    for cCity in pPlayer:Cities() do
        local pCityPlot = cCity:Plot()
        local between = Map.PlotDistance(pCityPlot:GetX(), pCityPlot:GetY(), pPlot:GetX(), pPlot:GetY())
        if between < distance then
            distance = between
            cNearestCity = cCity
        end
    end
    return cNearestCity
end
-------------------------------------------------------------------------------------------------------------------------
function IsBuildingWonder( buildingID )
	for row in DB.Query("SELECT a.ID buildingID, b.MaxGlobalInstances WWonderInt, b.MaxPlayerInstances NWonderInt FROM Buildings a, BuildingClasses b WHERE a.Type = b.DefaultBuilding AND a.ID = '" .. buildingID .. "'") do
		if row.WWonderInt <1 and row.NWonderInt <1 then
			return false
		else
			return true
		end
	end
end
-------------------------------------------------------------------------------------------------------------------------
function GetImprovementIDFromResourceID( resourceID)
	tResImprovements  = {}
	for row in DB.Query("SELECT a.ID ResID, b.ResourceType ResType, c.ID ImprovID FROM Resources a, Improvement_ResourceTypes b, Improvements c WHERE a.Type = b.ResourceType and b.ImprovementType = c.Type and a.ID = '" .. resourceID .. "'") do
		table.insert(tResImprovements, row.ImprovID);
	end
	return tResImprovements[1]
end
-------------------------------------------------------------------------------------------------------------------------
function GetResourceClassIDFromResID(resourceID)
	for row in DB.Query("SELECT a.ID ResID, b.ID ResClassID FROM Resources a, ResourceClasses b WHERE a.ResourceClassType = b.Type and a.ID = '" .. resourceID .. "'") do
		return row.ResClassID
	end
end
-------------------------------------------------------------------------------------------------------------------------
function GetTechRevealIDFromResID(resourceID)
	for row in DB.Query("SELECT b.ID TechID from Resources a, Technologies b WHERE a.TechReveal = b.Type and a.ID = '" .. resourceID .. "'") do
		return row.TechID
	end
end
-------------------------------------------------------------------------------------------------------------------------
if Game then
	--Game.GetUserSetting
	function Game.GetCustomModOptions(type)
		for row in GameInfo.CustomModOptions("Name = '" .. type .. "'") do
			return row.Value
		end
	end
end
-------------------------------------------------------------------------------------------------------------------------
function Jar_PlotHasBuildInProgress(plot)
	local bResult= false
	for i = 0, plot:GetNumUnits() - 1 do
		local pUnit = plot:GetUnit(i)
		local workrate = pUnit:WorkRate()
		local iCurrentBuildID = pUnit:GetBuildType()
		if pUnit and workrate>1 and iCurrentBuildID>-1 and plot:GetOwner() == pUnit:GetOwner() then
			bResult = true
		end
	end
	return bResult
end
--==========================================================================================================================
-- UNIQUE FUNCTIONS
--==========================================================================================================================
-- GLOBALS
--------------------------------------------------------------------------------------------------------------------------
local CityWorkingRadius = GameDefines.MAXIMUM_WORK_PLOT_DISTANCE
local iMod = ((GameInfo.GameSpeeds[Game.GetGameSpeedType()].BuildPercent)/100)
local iCiv = GameInfoTypes["CIVILIZATION_JAR_ZIMBABWE"]
local traitZimbabweID = GameInfoTypes["TRAIT_JAR_ZIMBABWE"]
local g_IsTraitActive = IsTraitActive(traitZimbabweID)
local missionVaNyai = MissionTypes.MISSION_JAR_KUTEMA_UGARIRI
local iPromoKutema  = GameInfoTypes["PROMOTION_JAR_KUTEMA"]
local iUBZimbabwe = GameInfoTypes.BUILDING_JAR_DZIMBABWE
local iDummyUB = GameInfoTypes.BUILDING_JAR_DZIMBABWE_DUM
local iDummy2UB = GameInfoTypes.BUILDING_JAR_DZIMBABWE_DUM2
local iPromoGunTrade = GameInfoTypes.PROMOTION_JAR_GUNTRADE
local iRambatemwa = GameInfoTypes.BUILDING_JAR_RAMBATEMWA
local iResWildlife = GameInfoTypes.RESOURCE_JAR_WILDLIFE
local iImprWildlife = GameInfoTypes.IMPROVEMENT_JAR_WILDLIFE

local CUSTOM_MISSION_NO_ACTION		 	= 0
local CUSTOM_MISSION_ACTION			 	= 1
local CUSTOM_MISSION_DONE            	= 2
local CUSTOM_MISSION_ACTION_AND_DONE 	= 3

local maxpower2 = 4
tTRslotDummies = {}
for power=0,maxpower2 do
	tTRslotDummies[power] = GameInfoTypes["BUILDING_JAR_DUMMY_FREE_TRROUTE_" .. tostring(power)]
end
------------------------------------------------------------------------------------------------------------------------
-- QUICK MOD BALANCING PANEL
------------------------------------------------------------------------------------------------------------------------
-- Here are the changeable values that determine the strength of the effects described in the design.
-- UA
local UAprodX = 3/4 			-- +1 Production per Era per different improved resource
local fTRCattle = 1/4 			-- +1 Trade Route slot per 4 owned improved Cattle
local fTRCattleMax = 8
-- UU
local vaNyaiLevel = 5
-- UB
local fDefenseToScience = 1/5 	-- 1 Science for every 5 Defense
-- UU2 
local PromoGunTradeFactor = 0.02
local PromoGunTradeMax = 0.2
-- UB2
local WildlifeSpawnRate = 3	-- 3% chance per turn to expand to adjacent forest/jungle tile
--========================================================================================================================
-- FUNCTIONS
--========================================================================================================================
-- ZIMBABWE UA
--========================================================================================================================
function Jar_AddTRslotDummy(pCity, integer)
	print("Jar_AddTRslotDummy: " .. pCity:GetName() .. "; ".. integer)
	local binary = basen(integer,2)
    local numcifre = string.len(tostring(binary))
    for i=-numcifre,-1 do
        if string.len(tostring(binary)) == -i and binary~=0 then
            pCity:SetNumRealBuilding(tTRslotDummies[-i-1], 1)
            binary = floor( binary - math.pow(10, -i-1) )
        else
            pCity:SetNumRealBuilding(tTRslotDummies[-i-1], 0)
        end
    end
end

function Jar_ZimbabweUA(iPlayer)
	local pPlayer = Players[iPlayer]
	if not HasTrait(pPlayer, traitZimbabweID) then return end
	if not pPlayer:IsAlive() then return end
	local iImprovedCattle= 0
	local tTextstring = {}
	for pCity in pPlayer:Cities() do
		tImprovedResources = {}
		for i = 0, pCity:GetNumCityPlots() - 1, 1 do
			local pPlot = pCity:GetCityIndexPlot(i)
			local iResPlot = pPlot:GetResourceType()
			local iImprPlot = pPlot:GetImprovementType()
			local bHasTechReveal = Teams[pPlayer:GetTeam()]:IsHasTech(GetTechRevealIDFromResID(iResPlot))
			if pPlot and pPlot:GetOwner() == iPlayer and iResPlot >-1 and pPlot:GetWorkingCity() == pCity and bHasTechReveal then
				if pPlot:IsCity() then
					tImprovedResources[iResPlot] = true
				else
					if iImprPlot >-1 then
						-- check if improved plot has the right improvement to connect the resource
						for row in DB.Query("SELECT a.ID ResID, b.ResourceType ResType, c.ID ImprovID FROM Resources a, Improvement_ResourceTypes b, Improvements c WHERE a.Type = b.ResourceType and b.ImprovementType = c.Type and a.ID = '" .. iResPlot .. "'") do
							if iImprPlot == row.ImprovID then
								tImprovedResources[iResPlot] = true
								if iResPlot == GameInfoTypes.RESOURCE_COW then
									iImprovedCattle = iImprovedCattle +1
								end
								break
							end
						end
					end
				end
			end
		end
		local iImprResDiversity = 0
		for k, v in pairs(tImprovedResources) do
			if v then iImprResDiversity= iImprResDiversity +1 end
		end
		local currentEra = pPlayer:GetCurrentEra()
		if iImprResDiversity >0 then
			if currentEra>= GameInfoTypes.ERA_INDUSTRIAL then UAprodX=1 end
			local numbonus= math.max(1, math.floor((1+ currentEra)*iImprResDiversity*UAprodX) )
			pCity:ChangeProduction(numbonus)
			local textstr = pCity:GetName() .. ": +" .. numbonus .. "[ICON_PRODUCTION] [NEWLINE]"
			if numbonus > 0 then
				table.insert(tTextstring, textstr)
			end
		end
	end
	----
	local pCapital = pPlayer:GetCapitalCity()
	local TRslots2add = math.min( math.floor(fTRCattle* iImprovedCattle) , fTRCattleMax)
	if TRslots2add > 0 then
		Jar_AddTRslotDummy(pCapital, TRslots2add)
	end
	----
	-- Notification
	local sTitle = "Zimbabwe UA: [ICON_PRODUCTION] Production from Resource Diversity"
	local sText1 = ""
	local sText2 = ""
	local icheck = 0
	local rowlimit= 10
	for k, v in pairs(tTextstring) do
		if k<= rowlimit then
			sText1 = sText1 .. k .. ") " .. v
		else
			sText2 = sText2 .. k .. ") " .. v
		end
		icheck = k
	end
	if pPlayer:IsHuman() and icheck>0 then
		pPlayer:AddNotification(NotificationTypes.NOTIFICATION_GENERIC, sText1, sTitle)
		if icheck> rowlimit then
			pPlayer:AddNotification(NotificationTypes.NOTIFICATION_GENERIC, sText2, sTitle)
		end
	end
end
if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(Jar_ZimbabweUA) end
--==========================================================================================================================
-- UU: VANYAI
--==========================================================================================================================
local tAllowedFeatures = {}
tAllowedFeatures[-1] = true
tAllowedFeatures[GameInfoTypes.FEATURE_JUNGLE] = true
tAllowedFeatures[GameInfoTypes.FEATURE_MARSH] = true
tAllowedFeatures[GameInfoTypes.FEATURE_FOREST] = true
tAllowedFeatures[GameInfoTypes.FEATURE_FLOOD_PLAINS] = true

function Jar_CanDoKutemaUgariri(unit) -- checks if a unit is on a valid plot
	local pBool = false;
	local pPlot = unit:GetPlot();
	if unit:MovesLeft() <= 1 then return pBool end
	if unit:GetLevel() < vaNyaiLevel then return pBool end
	local unitOwner = unit:GetOwner()
	if pPlot and (not pPlot:IsCity()) and (not pPlot:IsWater()) then
		local iOwner = pPlot:GetOwner()
		local pCity = Neirai_GetNearestCity(Players[unitOwner], pPlot)
		local distance = Map.PlotDistance(pCity:GetX(), pCity:GetY(), pPlot:GetX(), pPlot:GetY())
		if distance<= CityWorkingRadius and pPlot:GetResourceType() == -1 and tAllowedFeatures[pPlot:GetFeatureType()] then
			if iOwner == -1 or iOwner == unitOwner then
				pBool = true;
			end
		end
	end
	return pBool
end

function Jar_DoKutemaUgariri(pPlayer, pUnit)
	local pUnitPlot = pUnit:GetPlot();
	local plotownerID = pUnitPlot:GetOwner()
	local unitOwnerID = pUnit:GetOwner()
	local iTeam = pPlayer:GetTeam()
	local pCity = Neirai_GetNearestCity(pPlayer, pUnitPlot)
	if plotownerID== -1 then
		pUnitPlot:SetOwner(unitOwnerID, pCity:GetID(), true, true);
		pUnitPlot:SetRevealed(iTeam, true)
	end
	pUnitPlot:SetResourceType(GameInfoTypes.RESOURCE_COW, 1)
	pUnitPlot:SetImprovementType(GameInfoTypes.IMPROVEMENT_PASTURE)
	Game.SetPlotExtraYield(pUnitPlot:GetX(), pUnitPlot:GetY(), GameInfo.Yields.YIELD_CULTURE.ID, 1 )
	if  pUnitPlot:GetFeatureType() == GameInfoTypes.FEATURE_JUNGLE or
		pUnitPlot:GetFeatureType() == GameInfoTypes.FEATURE_FOREST or
		pUnitPlot:GetFeatureType() == GameInfoTypes.FEATURE_MARSH  then
		pUnitPlot:SetFeatureType(-1)
	end	
	pUnit:SetHasPromotion(iPromoKutema, false)	
end

-- Jar_Kurema_UgaririMissionStart
local function Jar_Kurema_UgaririMissionStart(playerID, unitID, missionID, data1, data2, flags, turn)
	local player = Players[playerID]
	local unit = player:GetUnitByID(unitID)
	local plot = unit:GetPlot()
	if missionID == missionVaNyai then
		Jar_DoKutemaUgariri(player, unit)
		return CUSTOM_MISSION_ACTION
	end
	return CUSTOM_MISSION_NO_ACTION
end

-- Jar_Kurema_UgaririMissionCompleted
local function Jar_Kurema_UgaririMissionCompleted(playerID, unitID, missionID, data1, data2, flags, turn)
	return (missionID == missionVaNyai)
end

-- Jar_Kurema_UgaririMissionPossible
local function Jar_Kurema_UgaririMissionPossible(playerID, unitID, missionID, data1, data2, _, _, plotX, plotY, bTestVisible)
	if ((not playerID) or (not unitID) or (not missionID)) then return end
	local player = Players[playerID]
	local unit = player:GetUnitByID(unitID)
	if missionID == missionVaNyai then
		if unit:IsHasPromotion(iPromoKutema) then
			if Jar_CanDoKutemaUgariri(unit) then
				return true
			else
				return bTestVisible
			end
		end
	end	
	return false
end

GameEvents.CustomMissionStart.Add(Jar_Kurema_UgaririMissionStart)
GameEvents.CustomMissionCompleted.Add(Jar_Kurema_UgaririMissionCompleted)
GameEvents.CustomMissionPossible.Add(Jar_Kurema_UgaririMissionPossible)

function Jar_AIDoPerformAction(playerID)
	local aPlayer = Players[playerID]
	local aTeam = Teams[aPlayer:GetTeam()]
	if aPlayer:IsHuman() then return end	
	if aPlayer:GetNumUnitPromotions(iPromoKutema) <1 then return end
	for aUnit in aPlayer:Units() do
		if aUnit:IsHasPromotion(iPromoKutema) and aUnit:GetLevel() >= vaNyaiLevel then
			local aUnitPlot = aUnit:GetPlot();
			local pCity = Neirai_GetNearestCity(aPlayer, aUnitPlot)
			local tEligiblePlots = {}
			for i = 0, pCity:GetNumCityPlots() - 1, 1 do
				local pPlot = pCity:GetCityIndexPlot(i)
				local distance = Map.PlotDistance(pCity:GetX(), pCity:GetY(), pPlot:GetX(), pPlot:GetY())
				if pPlot and distance <= CityWorkingRadius and pPlot:GetResourceType() == -1 and tAllowedFeatures[pPlot:GetFeatureType()] and (not pPlot:IsCity()) and (not pPlot:IsWater()) then
					if pPlot:GetOwner() == -1 or pPlot:GetOwner() == playerID then
						table.insert(tEligiblePlots, pPlot)
					end
				end
			end
			if (#tEligiblePlots > 0) then
				local randomdraw = GetRandom(1, #tEligiblePlots )
				local chosenPlot = tEligiblePlots[randomdraw]
				if chosenPlot:GetOwner() == -1 then
					chosenPlot:SetOwner(unitOwnerID, pCity:GetID(), true, true);
					chosenPlot:SetRevealed(aPlayer:GetTeam(), true)
				end
				chosenPlot:SetResourceType(GameInfoTypes.RESOURCE_COW, 1)
				chosenPlot:SetImprovementType(GameInfoTypes.IMPROVEMENT_PASTURE)
				Game.SetPlotExtraYield(chosenPlot:GetX(), chosenPlot:GetY(), GameInfo.Yields.YIELD_CULTURE.ID, 1 )
				aUnit:SetHasPromotion(iPromoKutema, false)
			end
		end
	end
end
GameEvents.PlayerDoTurn.Add(Jar_AIDoPerformAction)
--==========================================================================================================================
-- UB: ZIMBABWE
--==========================================================================================================================
function Jar_ZimbabweCastle(iPlayer)
	local pPlayer = Players[iPlayer]
	if not pPlayer:IsAlive() then return end
	if not HasTrait(pPlayer, traitZimbabweID) then return end
	if pPlayer:CountNumBuildings(iUBZimbabwe) <1 then return end
	local tCityTRcount = {}
	for pCity in pPlayer:Cities() do
		if pCity:IsHasBuilding(iUBZimbabwe) then
			tCityTRcount[pCity]= 0
		end
	end
	for _, route in ipairs(pPlayer:GetTradeRoutes()) do
		if route.FromID == iPlayer and route.FromCity:IsHasBuilding(iUBZimbabwe) then
			tCityTRcount[route.FromCity] = tCityTRcount[route.FromCity] +1
		end
		if route.ToID == iPlayer and route.ToCity:IsHasBuilding(iUBZimbabwe) then
			tCityTRcount[route.ToCity] = tCityTRcount[route.ToCity] +1
		end
	end
	for kCity, iNum in pairs(tCityTRcount) do
		kCity:SetNumRealBuilding(iDummyUB, iNum)
	end
	for pCity in pPlayer:Cities() do
		if pCity:IsHasBuilding(iUBZimbabwe) then
			local science2add = math.floor(fDefenseToScience* pCity:GetStrengthValue() *0.01 )
			pCity:SetNumRealBuilding(iDummy2UB, science2add)
			print("Zimbabwe UB, " .. pCity:GetName() .. ". " .. " City Defense from Trade Routes: +" .. tCityTRcount[pCity] .. "Science from City Defense: " .. science2add)
		end
	end
end
if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(Jar_ZimbabweCastle) end;
--==========================================================================================================================
-- UU2: ZVIGIDI
--==========================================================================================================================
function Jar_ZimbabweGunTrade(playerID)
	local pPlayer = Players[playerID]
	if pPlayer:GetNumUnitPromotions(iPromoGunTrade) <1 then return end
	if not pPlayer:IsAlive() then return end
	local pCapital = pPlayer:GetCapitalCity()
	local countIntTR = 0
	for _, route in ipairs(pPlayer:GetTradeRoutes()) do
		if route.FromID ~= route.ToID then
			countIntTR = countIntTR +1
		end
	end
	for pUnit in pPlayer:Units() do
		if pUnit:IsHasPromotion(iPromoGunTrade) then
			local strength = GameInfo.Units[pUnit:GetUnitType()].Combat
			local strength2add = math.min(countIntTR*PromoGunTradeFactor , PromoGunTradeMax)
			pUnit:SetBaseCombatStrength(strength *(1 + strength2add) )
		end
	end
end
if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(Jar_ZimbabweGunTrade) end
--==========================================================================================================================
-- UB2: RAMBATEMWA
--==========================================================================================================================
function Jar_ZimbabweUB2(iPlayer, iCity, iBuildingType, bGold, bFaithOrCulture)
	local pPlayer = Players[iPlayer];
	local pCity = pPlayer:GetCityByID(iCity)
	if not pPlayer:IsAlive() then return end
	if not HasTrait(pPlayer, traitZimbabweID) then return end
	if iBuildingType == iRambatemwa then
		local bcheck = false
		local chosenPlot 
		for i = 0, pCity:GetNumCityPlots() - 1, 1 do
			local pPlot = pCity:GetCityIndexPlot(i)
			local distance = Map.PlotDistance(pCity:GetX(), pCity:GetY(), pPlot:GetX(), pPlot:GetY())
			if pPlot and distance<= CityWorkingRadius and pPlot:GetOwner() == iPlayer and pPlot:GetResourceType() == -1 and pPlot:GetWorkingCity() == pCity and pPlot:GetImprovementType() < 0 and (not Jar_PlotHasBuildInProgress(pPlot)) then
				if pPlot:GetFeatureType() == GameInfoTypes.FEATURE_FOREST or pPlot:GetFeatureType() == GameInfoTypes.FEATURE_JUNGLE then
					bcheck = true
					chosenPlot = pPlot
					break
				end
			end
		end
		if bcheck then
			chosenPlot:SetResourceType(iResWildlife, 1)
			chosenPlot:SetImprovementType(iImprWildlife)
		else
			for i = 0, pCity:GetNumCityPlots() - 1, 1 do
				local zPlot = pCity:GetCityIndexPlot(i)
				local distance = Map.PlotDistance(pCity:GetX(), pCity:GetY(), zPlot:GetX(), zPlot:GetY())
				if zPlot and distance<= CityWorkingRadius and zPlot:GetOwner() == iPlayer and zPlot:GetResourceType() == -1 and zPlot:GetWorkingCity() == pCity  and zPlot:GetImprovementType() < 0 and (not Jar_PlotHasBuildInProgress(zPlot)) and (not zPlot:IsWater()) then
					local randomdraw = GetRandom(1, 100 )
					if randomdraw <=50 then
						zPlot:SetFeatureType(GameInfoTypes.FEATURE_FOREST)
					else
						zPlot:SetFeatureType(GameInfoTypes.FEATURE_JUNGLE)
					end
					zPlot:SetResourceType(iResWildlife, 1)
					zPlot:SetImprovementType(iImprWildlife)
					break
				end
			end
		end
	end
end
if g_IsTraitActive then GameEvents.CityConstructed.Add(Jar_ZimbabweUB2) end

function Jar_ZimbabweWildlifeExpand(iPlayer)
	local pPlayer = Players[iPlayer]
	if not pPlayer:IsAlive() then return end
	if not HasTrait(pPlayer, traitZimbabweID) then return end
	if pPlayer:CountNumBuildings(iRambatemwa) <1 then return end
	for pCity in pPlayer:Cities() do
		if pCity:IsHasBuilding(iRambatemwa) and pCity:CountResource(iResWildlife) >0 then
			local draw = GetRandom(1, 100)
			for i = 0, pCity:GetNumCityPlots() - 1, 1 do
				local zPlot = pCity:GetCityIndexPlot(i)
				if zPlot and zPlot:GetOwner() == iPlayer and zPlot:GetResourceType() == iResWildlife and zPlot:GetWorkingCity() == pCity then
					if zPlot:GetImprovementType() ~= iImprWildlife or zPlot:GetFeatureType() <0 then
						zPlot:SetResourceType(-1)
					end
					if draw <= WildlifeSpawnRate then
						for direction = 0, DirectionTypes.NUM_DIRECTION_TYPES - 1, 1 do
							local pAdjacentPlot = Map.PlotDirection(zPlot:GetX(), zPlot:GetY(), direction)
							if pAdjacentPlot and pAdjacentPlot:GetOwner() == iPlayer and pAdjacentPlot:GetResourceType() == -1 and pAdjacentPlot:GetWorkingCity() == pCity and pAdjacentPlot:GetImprovementType() < 0 and (not Jar_PlotHasBuildInProgress(pAdjacentPlot)) then
								if pAdjacentPlot:GetFeatureType() == GameInfoTypes.FEATURE_FOREST or pAdjacentPlot:GetFeatureType() == GameInfoTypes.FEATURE_JUNGLE then
									pAdjacentPlot:SetResourceType(iResWildlife, 1)
									pAdjacentPlot:SetImprovementType(iImprWildlife)
									-- Notification
									if pPlayer:IsHuman() then
										local sTitle = "Wildlife expands in " .. pCity:GetName() .. "!"
										local sText = "+1 " .. GameInfo.Resources[iResWildlife].IconString .. " " .. Locale.ConvertTextKey( GameInfo.Resources[iResWildlife].Description ) .. " in " .. pCity:GetName() .. "!"
										pPlayer:AddNotification(NotificationTypes.NOTIFICATION_DISCOVERED_LUXURY_RESOURCE, sText, sTitle, pAdjacentPlot:GetX(), pAdjacentPlot:GetY(), iResWildlife)									
									end
									break
								end
							end
						end
					end					
				end
			end
		end
	end
end
if g_IsTraitActive then GameEvents.PlayerDoTurn.Add(Jar_ZimbabweWildlifeExpand) end;
---------------------------------------------------------
print("Jarcast's Zimbabwe functions loaded")
--==========================================================================================================================
--==========================================================================================================================